import { html } from '@polymer/polymer/lib/utils/html-tag.js';
import { DemoReadyEventEmitter } from '@vaadin/vaadin-demo-helpers/vaadin-demo-ready-event-emitter.js';
import { PolymerElement } from '@polymer/polymer/polymer-element.js';
class GridBasicDemos extends DemoReadyEventEmitter(GridDemo(PolymerElement)) {
  static get template() {
    return html`
    <style include="vaadin-component-demo-shared-styles">
      :host {
        display: block;
      }
    </style>


    <h3>Basic Binding</h3>
    <p>
      Cells contents can be defined using <code>&lt;template&gt;</code> elements.
      Inside templates, variables like <code>[[index]]</code> and <code>[[item.prop]]</code>
      can be used to add dynamic content.
    </p>
    <p>
      <b>Note:</b> By default, the height of a row is determined by the height of its cell contents,
      as seen in this example.
    </p>
    <vaadin-demo-snippet id="grid-basic-demos-basic-binding">
      <template preserve-content="">
        <dom-bind>
          <template>
            <!-- Fetch an array of users to be shown in the grid -->
            <iron-ajax auto="" url="https://demo.vaadin.com/demo-data/1.0/people?count=200" handle-as="json" last-response="{{users}}"></iron-ajax>

            <!-- The array is set as the <vaadin-grid>'s "items" property -->
            <vaadin-grid aria-label="Basic Binding Example" items="[[users.result]]">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
                <!-- If necessary, the footer could be set using <template class="footer"> -->
                <template class="footer">#</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.firstName]]</template>
                <template class="footer">First Name</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.lastName]]</template>
                <template class="footer">Last Name</template>
              </vaadin-grid-column>

              <vaadin-grid-column width="8em">
                <template class="header">Address</template>
                <template>
                  <div style="white-space: normal">[[item.address.street]], [[item.address.city]]</div>
                </template>
                <template class="footer">Address</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
        </dom-bind>
      </template>
    </vaadin-demo-snippet>


    <h3>Two-Way Binding</h3>
    <p>
      Some template variables, like the item properties, can be modified using Polymer's two-way binding.
    </p>
    <vaadin-demo-snippet id="grid-basic-demos-two-way-binding">
      <template preserve-content="">
        <dom-bind>
          <template>
            <style>
              .vaadin-demo-input[readonly] {
                border: 2px solid transparent;
                pointer-events: none;
              }

              .vaadin-demo-input {
                font: inherit;
                width: 100%;
              }
            </style>

            <iron-ajax auto="" url="https://demo.vaadin.com/demo-data/1.0/people?count=200" handle-as="json" last-response="{{users}}"></iron-ajax>

            <vaadin-checkbox checked="{{editing}}" style="margin-bottom: 20px">Enable Editing</vaadin-checkbox>

            <vaadin-grid aria-label="Two-way Binding Example" items="[[users.result]]">

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>
                  <input class="vaadin-demo-input" value="{{item.firstName::input}}" readonly\$="[[!editing]]">
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>
                  <input class="vaadin-demo-input" value="{{item.lastName::input}}" readonly\$="[[!editing]]">
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column width="200px">
                <template class="header">Email</template>
                <template>[[item.firstName]].[[item.lastName]]@example.com</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
        </dom-bind>
      </template>
    </vaadin-demo-snippet>
`;
  }

  static get is() {
    return 'grid-basic-demos';
  }
}
customElements.define(GridBasicDemos.is, GridBasicDemos);
