import { html } from '@polymer/polymer/lib/utils/html-tag.js';
import { DemoReadyEventEmitter } from '@vaadin/vaadin-demo-helpers/vaadin-demo-ready-event-emitter.js';
import { PolymerElement } from '@polymer/polymer/polymer-element.js';
class GridDataDemos extends DemoReadyEventEmitter(GridDemo(PolymerElement)) {
  static get template() {
    return html`
    <style include="vaadin-component-demo-shared-styles">
      :host {
        display: block;
      }
    </style>


    <h3>Assigning Array Data</h3>
    <p>An array of objects can be assigned to the <code>items</code> property.</p>
    <vaadin-demo-snippet id="grid-data-demos-assigning-array-data">
      <template preserve-content="">
        <x-array-data-example></x-array-data-example>
        <dom-module id="x-array-data-example">
          <template preserve-content="">
            <vaadin-grid aria-label="Array Data Example" items="[[items]]">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.firstName]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.lastName]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-data-demos-assigning-array-data', function(document) {
              Polymer({
                is: 'x-array-data-example',

                ready: function() {
                  var items = [];

                  for (var i = 0; i < 100; i++) {
                    items.push({firstName: 'First Name ' + i, lastName: 'Last Name ' + i});
                  }

                  this.items = items;
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>


    <h3>Dynamic Array Data</h3>
    <p>
      The <code>items</code> property can be modified using Polymer's array mutation methods.
    </p>
    <vaadin-demo-snippet id="grid-data-demos-dynamic-array-data">
      <template preserve-content="">
        <x-dynamic-data-example></x-dynamic-data-example>
        <dom-module id="x-dynamic-data-example">
          <template preserve-content="">
            <style>
              vaadin-button {
                margin-bottom: 20px;
              }
            </style>
            <vaadin-button raised="" on-click="_add">Add Item</vaadin-button>
            <vaadin-button raised="" on-click="_remove">Remove Item</vaadin-button>

            <vaadin-grid aria-label="Dynamic Data Example" items="[[items]]">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.firstName]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.lastName]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-data-demos-dynamic-array-data', function(document) {
              Polymer({
                is: 'x-dynamic-data-example',

                ready: function() {
                  var items = [];

                  for (var i = 0; i < 10; i++) {
                    items.push({firstName: 'First Name ' + i, lastName: 'Last Name ' + i});
                  }

                  this.items = items;
                },

                _add: function() {
                  this.push('items', {firstName: 'First Name ' + this.items.length, lastName: 'Last Name ' + this.items.length});
                },

                _remove: function() {
                  this.pop('items');
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>


    <h3>Dynamic Height</h3>
    <p>
      Setting <code>height-by-rows</code> will make <code>&lt;vaadin-grid&gt;</code> grow
      and shrink in height depending on the number of rows.
    </p>
    <vaadin-demo-snippet id="grid-data-demos-dynamic-height">
      <template preserve-content="">
        <x-dynamic-height-example></x-dynamic-height-example>
        <dom-module id="x-dynamic-height-example">
          <template preserve-content="">
            <style>
              vaadin-button {
                margin-bottom: 20px;
              }
            </style>
            <vaadin-button raised="" on-click="_add">Add Item</vaadin-button>
            <vaadin-button raised="" on-click="_remove">Remove Item</vaadin-button>

            <vaadin-grid aria-label="Dynamic Data Example" items="[[items]]" height-by-rows="">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.firstName]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.lastName]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-data-demos-dynamic-height', function(document) {
              Polymer({
                is: 'x-dynamic-height-example',

                ready: function() {
                  var items = [];

                  for (var i = 0; i < 5; i++) {
                    items.push({firstName: 'First Name ' + i, lastName: 'Last Name ' + i});
                  }

                  this.items = items;
                },

                _add: function() {
                  this.push('items', {firstName: 'First Name ' + this.items.length, lastName: 'Last Name ' + this.items.length});
                },

                _remove: function() {
                  this.pop('items');
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>


    <h3>Assigning Remote/Function Data</h3>
    <p>
      A function can be assigned to the <code>dataProvider</code> property to provide
      data from a remote source.
    </p>
    <p>
      <b>Note:</b> the total number of items must be set to the grid. The example
      below sets the total number of items using the <code>size</code> property.
    </p>
    <p>
      Alternatively, the total size can be provided in the second argument of the
      data provider <code>callback</code>, see tree demo example.
    </p>
    <vaadin-demo-snippet id="grid-data-demos-assigning-remotefunction-data">
      <template preserve-content="">
        <x-remote-data-example></x-remote-data-example>
        <dom-module id="x-remote-data-example">
          <template preserve-content="">
            <vaadin-grid aria-label="Remote Data Example" data-provider="[[dataProvider]]" size="[[size]]">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.firstName]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.lastName]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-data-demos-assigning-remotefunction-data', function(document) {
              Polymer({
                is: 'x-remote-data-example',

                ready: function() {
                  this.size = 200;
                  this.dataProvider = function(params, callback) {
                    var xhr = new XMLHttpRequest();
                    xhr.onload = function() {
                      callback(JSON.parse(xhr.responseText).result);
                    };
                    var index = params.page * params.pageSize;
                    xhr.open('GET', 'https://demo.vaadin.com/demo-data/1.0/people?index=' + index + '&count=' + params.pageSize, true);
                    xhr.send();
                  };
                }

              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>
`;
  }

  static get is() {
    return 'grid-data-demos';
  }
}
customElements.define(GridDataDemos.is, GridDataDemos);
