import { html } from '@polymer/polymer/lib/utils/html-tag.js';
import { DemoReadyEventEmitter } from '@vaadin/vaadin-demo-helpers/vaadin-demo-ready-event-emitter.js';
import { PolymerElement } from '@polymer/polymer/polymer-element.js';
class GridPaginationDemos extends DemoReadyEventEmitter(GridDemo(PolymerElement)) {
  static get template() {
    return html`
    <style include="vaadin-component-demo-shared-styles">
      :host {
        display: block;
      }
    </style>

    <style>
      vaadin-grid {
        text-transform: none;
      }
    </style>


    <h3>Pagination</h3>
    <p>
      The <code>&lt;vaadin-grid&gt;</code> is an infinitely scrolling list by nature, but simple pagination can be implemented for example by changing the grid's <code>items</code> array on page change.
    </p>
    <vaadin-demo-snippet id="grid-pagination-demos-pagination">
      <template preserve-content="">
        <x-pagination></x-pagination>
        <dom-module id="x-pagination">
          <template preserve-content="">
            <style>
              #pages {
                display: flex;
                flex-wrap: wrap;
                margin: 20px;
              }

              #pages > button {
                user-select: none;
                padding: 5px;
                margin: 0 5px;
                border-radius: 10%;
                border: 0;
                background: transparent;
                font: inherit;
                outline: none;
                cursor: pointer;
              }

              #pages > button:hover,
              #pages > button:focus {
                color: #ccc;
                background-color: #eee;
              }

              #pages > button[selected] {
                font-weight: bold;
                color: white;
                background-color: #ccc;
              }
            </style>
            <x-array-data-provider items="{{items}}"></x-array-data-provider>

            <vaadin-grid id="grid" page-size="10" height-by-rows="">

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>
                  [[item.name.first]]
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>
                  [[item.name.last]]
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column width="200px">
                <template class="header">Email</template>
                <template>
                  <div>[[item.name.first]].[[item.name.last]]@example.com</div>
                </template>
              </vaadin-grid-column>

            </vaadin-grid>
            <div id="pages">
              <button on-click="_prev">&lt;</button>
              <template is="dom-repeat" items="[[pages]]">
                <button on-click="_select" selected\$="[[_isSelected(page, item)]]">[[item]]</button>
              </template>
              <button on-click="_next">&gt;</button>
            </div>
          </template>
          <script>
            window.addDemoReadyListener('#grid-pagination-demos-pagination', function(document) {
              Polymer({
                is: 'x-pagination',

                properties: {
                  items: Array,
                  page: Number,
                  pages: Array
                },

                ready: function() {
                  this.page = 0;
                },

                observers: ['_itemsChanged(items, page)'],

                _isSelected: function(page, item) {
                  return page === item - 1;
                },

                _select: function(e) {
                  this.page = e.model.item - 1;
                },

                _next: function() {
                  this.page = Math.min(this.pages.length - 1, this.page + 1);
                },

                _prev: function() {
                  this.page = Math.max(0, this.page - 1);
                },

                _itemsChanged: function(items, page) {
                  if (items === undefined || page === undefined) {
                    return;
                  }

                  if (!this.pages) {
                    this.pages = Array.apply(null, {length: Math.ceil(items.length / this.\$.grid.pageSize)}).map(function(item, index) {
                      return index + 1;
                    });
                  }

                  var start = page * this.\$.grid.pageSize;
                  var end = (page + 1) * this.\$.grid.pageSize;
                  this.\$.grid.items = items.slice(start, end);
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>
`;
  }

  static get is() {
    return 'grid-pagination-demos';
  }
}
customElements.define(GridPaginationDemos.is, GridPaginationDemos);
