import { html } from '@polymer/polymer/lib/utils/html-tag.js';
import { DemoReadyEventEmitter } from '@vaadin/vaadin-demo-helpers/vaadin-demo-ready-event-emitter.js';
import { PolymerElement } from '@polymer/polymer/polymer-element.js';
class GridSelectionDemos extends DemoReadyEventEmitter(GridDemo(PolymerElement)) {
  static get template() {
    return html`
    <style include="vaadin-component-demo-shared-styles">
      :host {
        display: block;
      }
    </style>


    <h3>Selection using Active Item</h3>
    <p>
      When a row is clicked, the item object from that row is assigned to <code>activeItem</code>.
    </p>
    <p>
      To select items on click, <code>activeItem</code> can be added
      to the <code>selectedItems</code> array. You can also call
      <code>selectItem(item)</code> or <code>deselectItem(item)</code> in order
      to select or deselect the grid item.
    </p>
    <p>
      In the example below, the <code>selectedItems</code> array is replaced
      whenever <code>activeItem</code> changes, making single-item selection.
    </p>
    <vaadin-demo-snippet id="grid-selection-demos-selection-using-active-item">
      <template preserve-content="">
        <x-selection></x-selection>
        <dom-module id="x-selection">
          <template preserve-content="">
            <x-data-provider data-provider="{{dataProvider}}"></x-data-provider>

            <vaadin-grid aria-label="Selection using Active Item Example" id="grid" data-provider="[[dataProvider]]" active-item="{{activeItem}}" size="200">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.name.first]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.name.last]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-selection-demos-selection-using-active-item', function(document) {
              Polymer({
                is: 'x-selection',

                properties: {
                  activeItem: {
                    observer: '_activeItemChanged'
                  }
                },

                _activeItemChanged: function(item) {
                  this.\$.grid.selectedItems = item ? [item] : [];
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>


    <h3>Selection using Templates</h3>
    <p>
      In addition to modifying selection using <code>selectedItems</code> property
      or <code>selectItem(item)</code> and <code>deselectItem(item)</code> methods,
      the template variable <code>{{selected}}</code> can also be used.
    </p>
    <vaadin-demo-snippet id="grid-selection-demos-selection-using-templates">
      <template preserve-content="">
        <dom-bind>
          <template>
            <x-data-provider data-provider="{{dataProvider}}"></x-data-provider>

            <vaadin-grid aria-label="Selection using Templates Example" data-provider="[[dataProvider]]" size="200">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.name.first]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.name.last]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template>
                  <vaadin-checkbox aria-label="Select Row" checked="{{selected}}">Selected</vaadin-checkbox>
                </template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
        </dom-bind>
      </template>
    </vaadin-demo-snippet>


    <h3>Multi-Selection Column</h3>
    <p>
      <code>&lt;vaadin-grid&gt;</code> ships with a helper element <code>&lt;vaadin-grid-selection-column&gt;</code>
      which provides a checkboxes bound to selection. It also comes with a "select all"
      checkbox for the header.
    </p>
    <p>
      <b>Hint: </b><code>auto-select</code> can be set to enable selecting rows on click.
    </p>
    <p>
      <b>Note: This is a opt-in feature so <code>vaadin-grid-selection-column.html</code> needs
      to be explicitly imported.</b>
    </p>
    <vaadin-demo-snippet id="grid-selection-demos-multi-selection-column">
      <template preserve-content="">
        <dom-bind>
          <template>

            <x-array-data-provider items="{{items}}"></x-array-data-provider>

            <vaadin-grid aria-label="Multi-Selection Example" items="[[items]]">
              <vaadin-grid-selection-column auto-select="">
              </vaadin-grid-selection-column>

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">#</template>
                <template>[[index]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.name.first]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.name.last]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
        </dom-bind>
      </template>
    </vaadin-demo-snippet>




    <h3>Custom Select All with Data Provider</h3>
    <p>
      When dealing with remote data, "selecting all" can be tricky since pushing
      every item to <code>selectedItem</code> would require fetching all of the items.
    </p>
    <p>
      In this example, a custom selection state is used to provide a "select all"
      functionality without having to fetch all of the items.
    </p>
    <vaadin-demo-snippet id="grid-selection-demos-custom-select-all-with-data-provider">
      <template preserve-content="">
        <dataprovider-select-all></dataprovider-select-all>
        <dom-module id="dataprovider-select-all">
          <template preserve-content="">
            <x-data-provider data-provider="{{dataProvider}}"></x-data-provider>

            <vaadin-grid aria-label="Select All with Data Example" id="grid" data-provider="[[dataProvider]]" inverted\$="[[inverted]]" size="200">

              <vaadin-grid-column width="60px" flex-grow="0">
                <template class="header">
                  <input aria-label="Select All" type="checkbox" on-click="_invert" checked="[[_isChecked(inverted, indeterminate)]]" indeterminate="[[indeterminate]]">
                </template>
                <template>
                  <input aria-label="Select Row" type="checkbox" on-change="_selectItem" checked="[[_isSelected(inverted, selected)]]">
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">First Name</template>
                <template>[[item.name.first]]</template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Last Name</template>
                <template>[[item.name.last]]</template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
          window.addDemoReadyListener('#grid-selection-demos-custom-select-all-with-data-provider', function(document) {
            Polymer({
              is: 'dataprovider-select-all',

              properties: {
                inverted: {
                  type: Boolean,
                  value: false
                },
                indeterminate: {
                  type: Boolean,
                  value: false
                }
              },

              observers: ['_resetSelection(inverted)'],

              _resetSelection: function(inverted) {
                this.\$.grid.selectedItems = [];
                this.updateStyles();
                this.indeterminate = false;
              },

              _invert: function(e) {
                this.inverted = !this.inverted;
              },

              // iOS needs indeterminate + checked at the same time
              _isChecked: function(inverted, indeterminate) {
                return indeterminate || inverted;
              },

              _selectItem: function(e) {
                if (e.target.checked === this.inverted) {
                  this.\$.grid.deselectItem(e.model.item);
                } else {
                  this.\$.grid.selectItem(e.model.item);
                }
                this.indeterminate = this.\$.grid.selectedItems.length > 0;
              },

              _isSelected: function(inverted, selected) {
                return inverted != selected;
              }
            });
          });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>


    <h3>Space Key Action and Click to Activate</h3>
    <p>
      When a focused cell has child elements, the <kbd>Space</kbd> key clicks
      the first child element.
    </p>
    <p>
      Clicking a child element inside a cell activates the item, unless either:
    </p>
    <ul>
      <li>
        The clicked child is a focusable element, for example,
        an <code>&lt;input&gt;</code>, a <code>&lt;button&gt;</code>, or has
        the <code>tabindex</code> attribute.
      </li>
      <li>The clicked child prevents default action of the <code>click</code> event.</li>
      <li>The clicked child stops propagation of the <code>click</code> event.</li>
    </ul>
    <p>
      Try clicking and pressing <kbd>Space</kbd> for the body cell contents
      in the example below.
    </p>
    <vaadin-demo-snippet id="grid-selection-demos-space-key-action-and-click-to-activate">
      <template preserve-content="">
        <x-space-action></x-space-action>
        <dom-module id="x-space-action">
          <template preserve-content="">
            <x-data-provider data-provider="{{dataProvider}}"></x-data-provider>

            <vaadin-grid aria-label="Space Key Action Example" id="grid" data-provider="[[dataProvider]]" active-item="{{activeItem}}" size="200">

              <vaadin-grid-column>
                <template class="header">Only text contents</template>
                <template>
                  Space activates
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Button first child</template>
                <template>
                  <button on-click="_alert">Button</button>
                  Space does not activate
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">Div first child</template>
                <template>
                  <div on-click="_alert">Div</div>
                  Space activates
                </template>
              </vaadin-grid-column>

              <vaadin-grid-column>
                <template class="header">preventDefault for click</template>
                <template>
                  <div on-click="_alertAndPreventDefault">Div with preventDefault</div>
                  Space does not activate
                </template>
              </vaadin-grid-column>

            </vaadin-grid>
          </template>
          <script>
            window.addDemoReadyListener('#grid-selection-demos-space-key-action-and-click-to-activate', function(document) {
              Polymer({
                is: 'x-space-action',

                properties: {
                  activeItem: {
                    observer: '_activeItemChanged'
                  }
                },

                _alert: function(e) {
                  alert(e.target.textContent + ' clicked');
                },

                _alertAndPreventDefault: function(e) {
                  e.preventDefault();
                  this._alert(e);
                },

                _activeItemChanged: function(item) {
                  this.\$.grid.selectedItems = item ? [item] : [];
                }
              });
            });
          &lt;/script>
        </dom-module>
      </template>
    </vaadin-demo-snippet>
`;
  }

  static get is() {
    return 'grid-selection-demos';
  }
}
customElements.define(GridSelectionDemos.is, GridSelectionDemos);
